#!/usr/bin/perl

# Script to turn a Perl program containing PODs into all PODs.  The
# idea is to try to create a nicely formatted internal document
# containing both the POD content and the Perl source code.  This is a
# quick-and-dirty attempt at literate programming.
#
# Note: this is NOT good code.
#


my $indent = "    ";	# Code indent string
my @text = ();			# The source text

# Suck in the source code, condensing any sequences of blank lines we
# may find.
do {
  my $blanks = 0;

  while (<>) {

	# Condense any sequence of more than one blank to one blank.
	++$blanks if /^\s*$/;
	$blanks = 0 if /\S/;
	next if $blanks > 1;

	s/\t/    /g;
	s/\s*$//;

	push @text, "$_\n";
  }
};

# Strip off all lines up until the first POD.
while ($text[0] !~ /^\=/) {
  shift @text;
}

# Strip off all trailing blank lines.
while ($text[-1] =~ /^\s*$/) {
  pop @text;
}


#
# Now, we do the real processing:
#


# Read in a POD and print it out again up until the final =cut.
sub write_pod {
  while (defined($_ = shift @text)) {
	print;
	last if /^=cut\s*$/;
  }

  print "\n";
}

# Given a block of comments, indent them all to the usual indent width
# and return the result.
sub comment_indent ( $ ) {
  my $comment = shift;

  $comment = $indent.$comment;
  $comment = s/\n\#/\n$indent\#/g;

  return $comment;
}

# Given a chunk of code, convert it to POD and print it.
sub podify_code {
  my $code = "";
  my $comment = "";

  while (defined($_ = shift @text)) {
	last if /^=/;

if (/^sub lforeach/) {
printf "splort\n";
}

	# If we are in the middle of a comment block, or at the end of
	# one...
	if ($comment ne "" || /^\#/) {

	  # If we're in a comment, just append it to $comment and go on to
	  # the next line.
	  if (/^\#/) {
		$comment .= $_;
		next;
	  } elsif (/^sub (\w+)/) {
		# If we've found the start of the sub, we need to turn the
		# comment into an ordinary POD paragraph.
		
		my $func_name = $1;

		$comment =~ s/^\#\s*//g;
		$comment =~ s/\n\#\s*/\n/g;
		$comment =~ s/(\$(\w+))/C\<$1\>/g;
		chomp $comment;

		$code .= "\n=head3 $func_name\n\n$comment\n\n";
		$comment = "";
	  } else {
		# If we've gotten here, this comment block doesn't precede a
		# function, so we need to put it back the way it was. 

		$code .= comment_indent ($comment);
		$comment = "";
	  }
	}

	$code .= "$indent$_";
  }

  if ($comment ne "") {
	# If we get here, it means that the code ended with a comment
	# block of the sort that we sometimes treat specially.
	$code .= comment_indent ($comment);
  }

  if ($#text < 0) {
	$code .= "$indent$_";
	$_="";
  }

  # Trim off leading and trailing blank lines.
#  $code =~ s/^\s*//;
  $code =~ s/\s*$//;
#  $code = $indent.$code;

  print "\n=pod\n\n";
  print $code;
  print "\n\n=cut\n\n";

  print $_;
}



while (scalar (@text)) {
  write_pod;
  podify_code;
}





















